use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy;

use Xyzzy::Spiffy -Base;
use Xyzzy::Database::Config -mixin;
use Xyzzy::Document::Config -mixin;

use IO::File;

sub set_application {
	my $class = $_[0];
	die "Missing argument to Application\n" unless defined $class;
	eval "use $class";
	die $@ if $@;
	bless $self, $class;
	$self->new;
}

field 'benchmark';

sub set_benchmark {
	$self->benchmark(!!$_[0]);
}

sub handle_configuration_item {
	my ($key, $values, $filename, $lineno) = @_;
	if(defined $key) {
		my $lkey = lc("set_$key");
		if($self->can($lkey)) {
			eval { $self->$lkey(@$values) };
			die "$filename:$lineno: $key: $@" if $@;
		} else {
			die "Unknown key in configuration file $filename:$lineno: $key\n";
		}
	}
}

sub include {
	foreach my $filename (@_) {
		my $fh = new IO::File($filename, '<:utf8')
			or die "Can't open configuration file '$filename': $!\n";;

		my ($ws, $lineno);
		my ($key, @values);

		while(my $line = $fh->getline) {
			next if ord($line) == 35; #
			$line =~ s/\s+$//;
			if($line eq '') {
				$ws = 1;
				next;
			}
			if($line =~ s/^\s//) {
				unless(defined $key) {
					my $lineno = $fh->input_line_number;
					warn "Line continuation before key at $filename:$lineno\n";
					next;
				}
				if($ws) {
					push @values, '';
					undef $ws;
				}
				push @values, $line;
			} else {
				$self->handle_configuration_item($key, \@values, $filename, $lineno);
				@values = ();
				$lineno = $fh->input_line_number;
				($key, my $val) = split(/\s+/, $line, 2);
				push @values, $val;
				undef $ws;
			}	
		}

		die "Can't read from configuration file '$filename': $!\n"
			unless $fh->eof;

		$self->handle_configuration_item($key, \@values, $filename, $lineno);

		$fh->close;
	}
}

*set_include = *include;

sub error {
	return new Xyzzy::Response(status => new Xyzzy::Status(@_));
}

sub handler {
	return new Xyzzy::Static(cfg => $self);
}

sub new {
	return $self if ref $self;
	return bless {}, $self;
}
