use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Database::Context;

use Xyzzy::Context -Base;

field 'db', -init => '$self->cfg->conn';

package Xyzzy::Database;

use Xyzzy::Handler -base;

use Xyzzy::Database::Config;

use DBI;

field 'db_type', -init => '$self->cfg->$field';
field 'db_name', -init => '$self->cfg->$field';
field 'db_user', -init => '$self->cfg->$field';
field 'db_pass', -init => '$self->cfg->$field';
field 'db_options', -init => '$self->cfg->$field';

*db_datasource = *Xyzzy::Database::Config::db_datasource;

field 'conn', -init => 'DBI->connect($self->db_datasource, $self->db_user, $self->db_pass, $self->db_options)';

sub db {
	return new Xyzzy::Database::Transaction(cfg => $self, txn => $self->conn);
}

sub handle {
	my $ctx = new Xyzzy::Database::Context(ctx => shift, cfg => $self);
	my $res = eval { super($ctx) };
	my $err = $@;
	my $db = $self->{conn};
	if($db && $db->{Executed}) {
		eval {
			$db->rollback;
			warn "rolling back database transaction\n"
				unless $err;
		};
		warn $@ if $@;
	}
	die $err if $err;
	return $res;
}
